// Copyright (c) Graham Relf, UK, 2021
// www.grelf.net
'use strict';

// Made lying flat. Rotate about x (facing NS) or y (not NS) to stand up
function Clock (xcentre, ycentre, zcentre, radius, dotRGB, backVertices, backRGB, isFacingNS)
{ Polygon.call (this, xcentre, ycentre, backVertices, [backRGB]);
  this.radius = radius;
  var rad12th = Math.PI / 6;
  for (var a = 0; a < TWO_PI; a += rad12th)
  { var dot = createHexagon (xcentre, ycentre, 0.1);
    dot.setColour (dotRGB.r, dotRGB.g, dotRGB.b);
    dot.translate (radius * Math.cos (a), radius * Math.sin (a), 0);
    this.faces [0].addDecoration (dot);
  }
  var r3 = radius * 0.3;
  var r6 = radius * 0.6;
  this.setHandAngles ();
  this.hrHand = new Polygon (xcentre, ycentre,
    [this.centre, this.centre.newOffsetPt (-0.1, r3, 0),
     this.centre.newOffsetPt (0, r6, 0), this.centre.newOffsetPt (0.1, r3, 0)]);
  this.hrHand.rotateZ (this.hrDegs);
  this.faces [0].addDecoration (this.hrHand);
  var r4 = radius * 0.4;
  var r8 = radius * 0.8;
  this.mnHand = new Polygon (xcentre, ycentre,
    [this.centre, this.centre.newOffsetPt (-0.1, r4, 0),
     this.centre.newOffsetPt (0, r8, 0), this.centre.newOffsetPt (0.1, r4, 0)]);
  this.mnHand.rotateZ (this.mnDegs);
  this.faces [0].addDecoration (this.mnHand);
  this.isFacingNS = isFacingNS;
  if (isFacingNS) this.rotateX (-90);
  else this.rotateY (-90);
  this.translate (0, 0, zcentre);
  this.movable = true;
}

Clock.prototype = Object.create (Polygon.prototype);
Clock.prototype.constructor = Clock;

Clock.prototype.setHandAngles = function ()
{ var now = new Date ();
  var mins = now.getMinutes ();
  this.hrDegs = ((now.getHours () % 12) + mins / 60) * 30;
  this.mnDegs = mins * 6;
};

Clock.prototype.update = function ()
{ var cz = this.centre.z;
  this.hrHand.translate (0, 0, -cz);
  this.mnHand.translate (0, 0, -cz);
  if (this.isFacingNS)
  { this.hrHand.rotateY (-this.hrDegs);
    this.mnHand.rotateY (-this.mnDegs);
    this.setHandAngles ();
    this.hrHand.rotateY (this.hrDegs);
    this.mnHand.rotateY (this.mnDegs);
  }
  else // EW
  { this.hrHand.rotateX (-this.hrDegs);
    this.mnHand.rotateX (-this.mnDegs);
    this.setHandAngles ();
    this.hrHand.rotateX (this.hrDegs);
    this.mnHand.rotateX (this.mnDegs);
  }
  this.hrHand.translate (0, 0, cz);
  this.mnHand.translate (0, 0, cz);
};