// Copyright (c) Graham Relf, UK, 2020
// www.grelf.net
// A demonstration of using the 2D graphics context of HTML5 to create animated
// 3D worlds from plain JavaScript without any other libraries or frameworks.
'use strict';

const RANGE = 150;

function run ()
{ world = new World (RANGE, {r:170, g:221, b:255}, true);//sky, foggy
  me = new Viewer (-1, -135, 10.5, -45);
  loadImage ("tree1", "im/wood04.png", SPRITE);
  loadImage ("tree2", "im/wood05.png", SPRITE);
  loadImage ("tree3", "im/wood06.png", SPRITE);
  loadImage ("tree4", "im/wood07.png", SPRITE);
  loadImage ("tulip", "im/u3933_Tulip_256sq.jpg", FACE);
  loadImage ("teledoor", "im/teledoor.png", SPRITE);
  loadImage ("teleframe", "im/teleframe.png", SPRITE);
  awaitLoaded ();
}

function awaitLoaded ()
{ if (world.allImsLoaded ())
  { buildWorld ();
    showButtons ();
    makeKeyTable ();
    animate ();
  }
  else setTimeout (awaitLoaded, 20);
}

function buildWorld ()
{ for (var x = -RANGE; x <= RANGE; x += 10)
  { for (var y = -RANGE; y <= RANGE; y += 10)
    { if (!(x === 80 && y === 100)) // Trap in house
      { var turf = createRectangle (x, y, 5, 5);
        if (x >= 60 && x <= 110 && y >= 80 && y <= 110) turf.setColour (150, 150, 150);
        else turf.setColour (0, 128, 0);
        turf.setStroke (100, 100, 100);
        turf.background = true;
        world.add (turf);
  } } }
  world.trees = []; // point obstacles: trees, wall corners
  // Trees should be at least cleaner's diameter apart
  for (var i = 0; i < 7; i++)
  { addTree (50 + 10 * i, 50, 'tree1');
    addTree (50 + 10 * i, -60, 'tree2');
    addTree (55 + 6 * i, -40 + 8 * i, 'tree3');
    addTree (-50 + 7 * i, -50 - 7 * i, 'tree4');
    addTree (random (-100, -70), random (-100, -70), imFromN (Math.round (random (0, 4))));
  }
  // Under trap in house:
  var ds = 5; // Hole side enlargement
  var wds = 10 + 2 * ds;
  var side = createWall (75 - ds, 95 - ds, -15, 0, wds, 15);
  side.under = true;
  world.add (side);
  side = createWall (75 - ds, 105 + ds, -15, wds, 0, 15);
  side.under = true;
  world.add (side);
  side = createWall (85 + ds, 95 - ds, -15, 0, wds, 15);
  side.under = true;
  world.add (side);
  side = createWall (75 - ds, 95 - ds, -15, wds, 0, 15);
  side.under = true;
  world.add (side);
  var ladder = new Ladder ({x:80, y:105, z:-20}, 3, 20, 8, {r:180, g:150, b:50});
  ladder.under = true;
  world.add (ladder);
  var colourCube = 
  [ {r:255, g:0, b:0}, // bottom
    {r:0, g:255, b:255}, // top
    {r:0, g:255, b:0}, // N
    {r:255, g:0, b:255}, // S
    {r:0, g:0, b:255}, // E
    {r:255, g:255, b:0}  // W
  ];
  var cube1 = new Cuboid (-110, 180, 18, 5, 5, 5, colourCube);
  cube1.setAngular (0.5, 1.1, -1.5);
  cube1.faces [5].setImage ("tulip");
  world.add (cube1);
  var cube2 = new Cuboid (5, -40, 25, 3, 4, 8, colourCube);
  cube2.setVelocity (-0.02, -0.03, 0);
  cube2.setAngular (0.9, -2.1, 1.5);
  cube2.setStroking (false);
  cube2.explosive = true;
  world.add (cube2);
  var leaf1 = createRectangle (-20, 15, 5, 5);
  leaf1.setColour (128, 128, 0);
  leaf1.faces [0].setAlpha (0.3);
  leaf1.rotateY (90);
  leaf1.translate (0, 0, 5);
  leaf1.setVelocity (0.01, 0.03, 0);
  leaf1.setAngular (2, 4, 6);
  leaf1.isTarget = true;
  leaf1.response = function () { this.explode (); hint ("BANG!"); };
  world.add (leaf1);
  var leaf2 = createHexagon (15, -80, 5);
  leaf2.setColour (128, 128, 0);
  leaf2.rotateY (90);
  leaf2.translate (0, 0, 10);
  leaf2.setVelocity (-0.02, 0.03, 0);
  leaf2.setAngular (4, 5, 2);
  leaf2.setStroking (false);
  leaf2.explosive = true;
  leaf2.isTarget = true;
  leaf2.response = function () { this.explode (); hint ("BANG!"); };
  world.add (leaf2);
  var leaf3 = createStar (0, 0, 5, 3, 7);
  leaf3.setColour (128, 128, 0);
  leaf3.rotateY (90);
  leaf3.translate (0, 0, 15);
  leaf3.setVelocity (0.03, -0.04, 0);
  leaf3.setAngular (2, 3, 4);
  leaf3.isTarget = true;
  leaf3.response = function () { this.explode (); hint ("BANG!"); };
  world.add (leaf3);
  world.house = new House (85, 95, 60, 40, 20, 10); 
  world.add (world.house);
  // Mark corners as obstacles:
  world.trees.push ({x:85 - 30, y:95 - 20, z:0});
  world.trees.push ({x:85 + 30, y:95 - 20, z:0});
  world.trees.push ({x:85 - 30, y:95 + 20, z:0});
  world.trees.push ({x:85 + 30, y:95 + 20, z:0});
  world.trapdoor = new Door (new Point3D (75, 105, 0), new Point3D (85, 105, 0), new Vector3D (0, -10, 0),
    true, [{r:150, g:100, b:0}]);
  world.add (world.trapdoor);
  var clock = new Clock (0, 3, 0, 2, {r:0, g:0, b:0},
    [new Point3D (-3, 6.5, 0), new Point3D (3, 6.5, 0),
     new Point3D (3, -6.5, 0), new Point3D (-3, -6.5, 0)], 
    {r:180, g:180, b:180}, true);
  clock.translate (30, 150, 10);
  world.add (clock);
  me.score = function () // Called at end of forward move
  { if (!world.house.outside (this.pt))
    { if (!this.houseScored) 
      { world.score += 1000; this.houseScored = true; hint ("Entered house: +1000 points"); }
      if (!world.trapdoor.closed 
       && this.pt.x > 75 && this.pt.x < 85 && this.pt.y > 95 && this.pt.y < 105)
      { var cWd = world.canvas.width, cHt = world.canvas.height;
        var nSteps = 100, dy = Math.round (cHt / nSteps);
        world.uiLocked = true;
        world.animPaused = true;
        world.animData = { y:0, dy:dy, imData:world.g2.getImageData (0, 0, cWd, cHt) };
        var ladder = new Ladder ({x:80, y:105, z:-80}, 3, 60, 24, {r:180, g:150, b:50});
        ladder.under = true;
        world.add (ladder); // extension
        trapfall ();
    } }
    else // I am outside
    { for (var i = 0; i < world.bods.length; i++)
      { if (!world.house.outside (world.bods [i].centre)) return; } // Some inside
      if (!world.allOut)
      { world.allOut = true; world.score += 1000; hint ("All outside: +1000 points"); }
    }
  };
  var cube3 = new Cuboid (0, 0, 5, 5, 5, 5);
  cube3.setSolid ();
  cube3.isTarget = true;
  cube3.response = function () 
  { if (cube3.centre.distance (me.pt) < 30) moveAway (cube3);
    else hint ("Get closer?");
  };
  cube3.setCornersAsTrees ();
  world.add (cube3);
  var cube4 = new Cuboid (25, 10, 11, 10, 1, 7);
  cube4.rotateY (30);
  cube4.setSolid ();
  cube4.explosive = true;
  world.add (cube4);
  var cube5 = new Cuboid (-15, 0, 11, 1, 1, 11);
  cube5.rotateX (30);
  cube5.explosive = true;
  world.add (cube5);
  var hex = createHexagon (20, 60, 5);
  var rainbow = 
[
{r:96, g:96, b:96},//bottom
{r:96, g:96, b:96},//top
{r:142, g:0, b:142},//8e008e
{r:65, g:105, b:225},//4169e1
{r:0, g:142, b:0},//008e00
{r:255, g:255, b:0},//ff0
{r:255, g:142, b:0},//ff8e00
{r:255, g:0, b:0}//f00
];
  var prism1 = new Prism (20, 60, 5, 5, hex.vertices, rainbow);
  prism1.explosive = true;
  world.add (prism1);
  for (i = 0; i < 5; i++)
  { var y = -80 - i * 9;
    var column = createStar (100, y, 11, 2, 2.25).extrude (30);
    world.trees.push ({x:100, y:y, z:0});
    column.setColour (190, 190, 190);
    column.setStroke (100, 100, 100);
    column.translate (0, 0, -1);
    world.add (column);
    var cap = createRectangle (100, y, 4, 4.5).extrude (1);
    cap.setColour (190, 190, 190);
    cap.setStroke (100, 100, 100);
    cap.translate (0, 0, 29);
    cap.isTarget = true;
    cap.response = function () { this.explode (); hint ("BANG!"); };
    world.add (cap);
  }
  world.cogs = [];
  makeCog (-130, -100, 10, 2.7, 3.3, 0.5, 0.5, true); // Advance to mesh with cog 2
  makeCog (-130, -106, 10, 2.7, 3.3, 0.5, -0.5, false);
  makeCog (-140, -100, 10, 2.7, 3.3, 0.5, 2.0, false);
  makeCog (110, 100, 10, 2.7, 3.3, 0.5, -0.5, false);
  var tetra = new Tetrahedron (-100, 20, 5, 5);
  tetra.rotateY (45);
  tetra.isTarget = true;
  tetra.response = function () 
  { if (this.side < 20) 
    { this.scale (1.1); this.side *= 1.1; 
      this.translate (0, 0, -this.vertices [1].z + 5);
      hint ('Stretching');
    }
    else { this.explode (); hint ('BANG'); }
  };
  world.add (tetra);
  var octa = new Octahedron (-120, 20, 8, 15);
  octa.setAngular (0, 0, -0.2);
  octa.isTarget = true;
  octa.response = function ()
  { hint ("");
    switch (Math.round (random (0.5, 4.5)))
    { case 1: this.translate (0, 0, 5); return;
      case 2: if (this.centre.z > this.side) this.translate (0, 0, -5); return;
      case 3: this.angular.dz += 0.2; return;
      case 4: this.angular.dz -= 0.2; return;
    }
  };
  world.add (octa);
  var icosaColours = [];
  for (i = 0; i < 20; i++)
  { icosaColours.push ({r:random (60, 255), g:random (60, 255), b:random (60, 255)}); }
  var icosa = new Icosahedron (-100, 50, 15, 10, icosaColours);
  icosa.setAngular (0.07, 0.09, 0.11);
  icosa.isTarget = true;
  icosa.response = function (mouseX, mouseY)
  { var d2 = 10000000, n;
    var sxy0 = me.getScreenXYD (this.centre);
    var d20 = sxy0.d * sxy0.d;
    for (var i = 0; i < this.faces.length; i++)
    { var sxy = me.getScreenXYD (this.faces [i].centre);
      if (sxy.d * sxy.d < d20) // => a visible face
      { var dx = mouseX - sxy.x, dy = mouseY - sxy.y;
        var dd = dx * dx + dy * dy;
        if (dd < d2) { d2 = dd; n = i; }
    } }
    if (n) 
    { this.faces [n].setColour (random (60, 255), random (60, 255), random (60, 255));
      hint ("Recoloured a face");
    }
  };
  world.add (icosa);
  var dodeColours = [];
  for (i = 0; i < 12; i++)
  { dodeColours.push ({r:random (120, 255), g:random (120, 255), b:random (120, 255)}); }
  var dodec = new Dodecahedron (-120, 0, 5, 5, dodeColours);
  dodec.setVelocity (0.2, 0.1, 0.75);
  dodec.setAngular (1, 5, 3);
  dodec.setFreeFall (true);
  dodec.setStroking (false);
  dodec.bouncy = true;
  dodec.oldTranslate = dodec.translate;
  dodec.translate = function (dx, dy, dz) // Override
  { this.oldTranslate (dx, dy, dz);
    if (world.house.outside (this.centre)) return;
    this.setVelocity (-dx, -dy, -dz);
    this.oldTranslate (-dx, -dy, -dz);
  };
  dodec.isTarget = true;
  dodec.response = function () { this.velocity.dz += 0.25; hint ("How high can a dodecahedron go?"); };
  world.add (dodec);
  var slab = new Cuboid (149, 0, 10, 1, 5, 10);
  slab.setColour (0, 0, 0);
  slab.setStroke (100, 100,100);
  slab.setSolid ();
  slab.isTarget = true;
  slab.response = function () 
  { if (slab.centre.distance (me.pt) < 30) teleStart (this);
    else hint ("Get closer?");
  };
  world.add (slab);
  var bird1 = new Bird (1);
  bird1.translate (-100, -60, 32);
  bird1.setVelocity (0.12, -0.06, 0);
  bird1.rotateZ (30);
  bird1.setStroking (false);
  world.add (bird1);
  var bird2 = new Bird (1.4);
  bird2.translate (-100, 0, 34);
  bird2.setVelocity (0.15, 0.15, 0);
  bird2.rotateZ (-45);
  bird2.setStroking (false);
  world.add (bird2);
  var bird3 = new Bird (1.7);
  bird3.translate (-100, -10, 33);
  bird3.setVelocity (0.15, -0.15, 0);
  bird3.rotateZ (45);
  bird3.setStroking (false);
  world.add (bird3);
  world.cleaner = createCleaner (-50, -135, 5, 1.5);
  world.add (world.cleaner);
  var doorWd = 4, doorHt = 10, hingeX = 0, hingeY = -150;
  var door = new Door (new Point3D (hingeX, hingeY, 0), new Point3D (hingeX, hingeY, doorHt),
                       new Vector3D (doorWd, 0, 0), false, [{r:150, g:50, b:0}]);
  // Door frame:
  var TH = 0.1;
  world.add (createRectangle (-TH, hingeY, TH, TH).extrude (doorHt));
  world.add (createRectangle (doorWd + TH, hingeY, TH, TH).extrude (doorHt));
  var beam = createRectangle (hingeX, hingeY, TH, TH).extrude (doorWd + 4 * TH);
  beam.rotateY (90);
  beam.translate (doorWd / 2, 0, doorHt - doorWd / 2 - TH);
  world.add (beam);
  world.add (door);
  var rover = new Rover (-120, -140);
  rover.setVelocity (0.1, 0, 0);
  rover.addToWorld ();
  world.bods = [createBod (40, 140, 5, 180), createBod (108, 82, 5, 270)];
  for (i = 0; i < world.bods.length; i++) world.add (world.bods [i]);
  var ladder = new Ladder ({x:-100, y:-50, z:0}, 3, 20, 8, {r:180, g:150, b:50});
  ladder.rotateZ (45);
  world.add (ladder);
  world.trees.push ({x:ladder.centre.x, y:ladder.centre.y, z:0});
  plantDaisies ();
  var domino = new Domino (-30, 80, 0.5, 10, 5, 6, {r:0, g:0, b:0}, {r:255, g:255, b:255});
  domino.isTarget = true;
  domino.response = function ()
  { if (Math.random () > 0.5)
    { this.translate (random (-5, 5), random (-5, 5), 0); hint (''); }
    else
    { var no1 = Math.round (random (-0.49, 6.49)), no2 = Math.round (random (-0.49, 6.49));
      this.changePips (no1, no2);
      if (no1 === no2)
      { if (no1 === 6) var dScore = 500;
        else dScore = 50;
        hint (dScore + ' points for double ' + no1);
        world.score += dScore;
      }
      else hint ('');
    }
  };
  world.add (domino);
  var die = new Die (-10, -100, 1.5, 3, {r:0, g:0, b:255}, {r:255, g:255, b:0});
  die.isTarget = true;
  die.response = function () 
  { this.toss (random (-0.4, 0.4), random (-0.4, 0.4), random (0.1, 0.5),
               random (-5, 5), random (-5, 5), random (-5, 5)); 
  };
  world.add (die);
  var cubeA = new Cuboid (-4, 0, 20, 1, 1, 1);
  var cubeB = new Cuboid (4, 0, 20, 1, 1, 1);
  var pair = new Composite ([cubeA, cubeB]);
  pair.setStroking (false);
  pair.setAngular (0, 0, 0.6);
  world.add (pair);
  var coil = new Coil ({x:-50, y:-100, z:2}, 1, 4, 6, 16, {r:0, g:0, b:0});
  var cc = new Cuboid (-50, -100, 4.8, 1, 1, 1);
  coil.attach (cc);
  coil.movable = true;
  coil.update = function ()
  { var f6 = world.frameCount % 600;
    if (f6 < 300) this.spring (0.995);
    else this.spring (1 / 0.995);
  };
  cc.coil = coil;
  cc.isTarget = true;
  cc.response = function () { this.coil.movable = !this.coil.movable; };
  world.add (cc);
  world.add (coil);
  world.trees.push ({x:coil.centre.x, y:coil.centre.y, z:0});
}

function plantDaisies ()
{ var planted = [];
  for (var i = 0; i < 7; i++)
  { var tries = 0;
    do
    { var x = Math.round (random (-10, -15)) * 5;
      var y = 150 - Math.round (random (2, 6)) * 5;
      var already = false;
      for (var j = 0; j < planted.length; j++)
      { if (planted [j].x === x && planted [j].y === y) already = true; }
      tries++;
    } while (already && tries < 50);
    planted.push ({x:x, y:y});
    world.add (new Daisy (x, y, 1.5));
} }
    
function trapfall ()
{ me.pt.z -= 0.2;
  world.draw ();
  if (me.pt.z > -20) requestAnimationFrame (trapfall);
  else
  { world.animPaused = false;
    world.uiLocked = false;
    hint ("You can escape<br>if you have not<br>moved something too far")
    animate ();
} }

function makeCog (xCentre, yCentre, nTeeth, innerRadius, outerRadius, thickness, angularZ, advance)
{ var cog = createStar (xCentre, yCentre, nTeeth, innerRadius, outerRadius).extrude (thickness);
  cog.setColour (128, 128, 0);
  cog.setAngular (0, 0, angularZ);
  if (advance) cog.rotateZ (180 / nTeeth);
  world.add (cog);
  world.cogs.push (cog);
  var cover = createRegularPolygon (xCentre, yCentre, nTeeth, innerRadius);
  cover.setStroking (false); // Don't see it - just a target
  cover.isTarget = true;
  cover.response = reverseCogs;
  world.add (cover);
  var spindle = createHexagon (xCentre, yCentre, 0.1).extrude (1.5);
  spindle.translate (0, 0, thickness);
  spindle.setAngular (0, 0, angularZ);
  world.add (spindle);
}

function reverseCogs () // response
{ for (var i = 0; i < world.cogs.length; i++)
  { world.cogs [i].angular.reverse (); hint ("Are the cogs all connected?"); }
}

function moveAway (cube) // response
{ cube.removeCornersAsTrees ();
  var vec = cube.centre.difference (me.pt);
  cube.translate (vec.dx, vec.dy, 0);
  for (var i = 0; i < world.solidWalls.length; i++)
  { if (world.solidWalls [i].face.parent === cube) 
    { world.solidWalls.splice (i, 1);
      i--; // Not just removing 1 face
  } }
  cube.setSolid (); // Must redo this after each move
  cube.setCornersAsTrees ();
  hint ("Don't push the cube off the edge.<br>It could score you 1,000 points");
  if (!world.cube3Done && !world.house.outside (cube.centre))
  { world.score += 1000;
    world.cube3Done = true;
} }

function addTree (x, y, id)
{ world.add (new Sprite (x, y, id));
  world.trees.push ({x:x, y:y, z:0});
}

Cuboid.prototype.removeCornersAsTrees = function ()
{ var v = this.vertices;
  for (var i = 0; i < v.length; i++)
  { for (var j = world.trees.length - 1; j >= 0; j--)
    { var wtj = world.trees [j];
      if (wtj.x === v [i].x && wtj.y === v [i].y) world.trees.splice (j, 1);
  } }
};

Cuboid.prototype.setCornersAsTrees = function ()
{ var v = this.vertices;
  for (var i = 0; i < v.length; i++) world.trees.push (v [i]);
};

function imFromN (n) // For the random tree placement
{ switch (n)
  {
case 0: return 'tree1';
case 1: return 'tree2';
case 2: return 'tree3';
default: return 'tree4';
} }