// Copyright (c) Graham Relf, UK, 2020
// www.grelf.net
'use strict';

function House (centreX, centreY, width, depth, height, roofHt)
{ this.wd2 = width / 2; this.dp2 = depth / 2; this.ht2 = height / 2;
  this.totalHt = height + roofHt;
  this.centre = new Point3D (centreX, centreY, this.ht2);
  this.movable = false;
  this.parts = [];
  var wall, rh2 = roofHt / 2;
  var nWd = Math.round (width / 5), nWd2 = Math.round (nWd / 2);
  var subWd = width / nWd, subWd2 = subWd / 2;
  var x0 = centreX - this.wd2 + subWd2;
  for (var i = 0; i < nWd; i++, x0 += subWd)
  { if (i !== nWd2)
    { wall = createRectangle (x0, centreY - this.dp2, subWd2, this.ht2); // S
      wall.translate (0, 0, this.ht2);
    }
    else 
    { var ydp2 = centreY - this.dp2;
      wall = createRectangle (x0, ydp2, subWd2, this.ht2 * 0.333);
      wall.translate (0, 0, this.ht2 * 1.667);
      this.door = new Door (new Point3D (x0 - subWd2, ydp2, 0),
        new Point3D (x0 - subWd2, ydp2, this.ht2 * 1.333), new Vector3D (subWd, 0, 0), true,
        [{r:0, g:50, b:150}]);
      this.parts.push (this.door);
      world.add (this.door);
    }
    wall.setColour (255, 255, 255);
    wall.setStroke (255, 255, 255);
    wall.rotateX (90);
    if (i != nWd2) wall.setSolid ();
    this.parts.push (wall);
    world.add (wall);
    wall = createRectangle (x0, centreY + this.dp2, subWd2, this.ht2); // N
    wall.setColour (128, 128, 128);
    wall.setStroke (128, 128, 128);
    wall.rotateX (90);
    wall.translate (0, 0, this.ht2);
    wall.setSolid ();
    this.parts.push (wall);
    world.add (wall);
    var dp4 = this.dp2 / 2;
    var roof = new Prism (0, 0, 0, subWd2, // S
    [ { x:rh2 - 1, y:dp4},
      { x:-rh2, y:-dp4},
      { x:-rh2, y:-dp4 - 1},
      { x:rh2, y:dp4}
    ]);
    roof.rotateZ (90);
    roof.rotateY (-90);
    roof.rotateX (90);
    roof.translate (x0, centreY - dp4, height + rh2);
    roof.setColour (100, 100, 100);
    this.parts.push (roof);
    world.add (roof);
    roof = new Prism (0, 0, 0, subWd2, // N
    [ { x:rh2 - 1, y:-dp4},
      { x:-rh2, y:dp4},
      { x:-rh2, y:dp4 + 1},
      { x:rh2, y:-dp4}
    ]);
    roof.rotateZ (90);
    roof.rotateY (-90);
    roof.rotateX (90);
    roof.translate (x0, centreY + dp4, height + rh2);
    roof.setColour (100, 100, 100);
    this.parts.push (roof);
    world.add (roof);
  }
  var nDp = Math.round (depth / 5), subDp = depth / nDp, subDp2 = subDp / 2;
  var y0 = centreY - this.dp2 + subDp2;
  var nDp2 = Math.round (nDp * 0.5);
  for (i = 0; i < nDp; i++, y0 += subDp)
  { wall = createRectangle (centreX - this.wd2, y0, this.ht2, subDp2); // W
    if (i === nDp2)
    { wall.faces [0].setAlpha (0.6); // Glass
      wall.setColour (0, 192, 192);
      wall.isTarget = true;
      wall.response = function () {}; // Prevent actions through glass
    }
    else
    { wall.setColour (192, 192, 192);
      wall.setStroke (192, 192, 192);
    }
    wall.rotateY (90);
    wall.translate (0, 0, this.ht2);
    wall.setSolid ();
    this.parts.push (wall);
    world.add (wall);
    wall = createRectangle (centreX + this.wd2, y0, this.ht2, subDp2); // E
    wall.setColour (192, 192, 192);
    wall.setStroke (192, 192, 192);
    wall.rotateY (90);
    wall.translate (0, 0, this.ht2);
    wall.setSolid ();
    this.parts.push (wall);
    world.add (wall);
  }
  var gableX = centreX - this.wd2; // W
  var gable = new Polygon (gableX, centreY,
    [ {x:gableX, y:centreY + this.dp2, z:height},
      {x:gableX, y:centreY, z:height + roofHt - 1},
      {x:gableX, y:centreY - this.dp2, z:height}
    ]);
  gable.setColour (192, 192, 192);
  gable.setStroking (false);
  this.parts.push (gable);
  world.add (gable);
  gableX = centreX + this.wd2; // E
  gable = new Polygon (gableX, centreY,
    [ {x:gableX, y:centreY + this.dp2, z:height},
      {x:gableX, y:centreY, z:height + roofHt - 1},
      {x:gableX, y:centreY - this.dp2, z:height}
    ]);
  gable.setColour (192, 192, 192);
  gable.setStroking (false);
  this.parts.push (gable);
  world.add (gable);
}

/* For consistency. Do nothing <= parts all drawn by world */
House.prototype.view = function (me, fogNo) {};

House.prototype.over = function (pt3d, margin)
{ return pt3d.x >= this.centre.x - this.wd2 - margin 
    && pt3d.x <= this.centre.x + this.wd2 + margin
    && pt3d.y >= this.centre.y - this.dp2 - margin
    && pt3d.y <= this.centre.y + this.dp2 + margin;
};

House.prototype.outside = function (pt3d)
{ return pt3d.x < this.centre.x - this.wd2 || pt3d.x > this.centre.x + this.wd2
      || pt3d.y < this.centre.y - this.dp2 || pt3d.y > this.centre.y + this.dp2
      || pt3d.z > this.totalHt;
};