// Copyright (c) Graham Relf, UK, 2020
// www.grelf.net
'use strict';

function Rover (centreX, centreY)
{ this.centre = new Point3D (centreX, centreY, 1);
  var axisVec = new Vector3D (0, 1, 0);
  var wheelFill = {r:160, g:160, b:160};
  var wheelStroke = {r:0, g:0, b:0};
  this.wheels =
  [ new Wheel (this.centre.newOffsetPt (4, 3, 0), 1, axisVec, wheelFill, wheelStroke, 0.1),
    new Wheel (this.centre.newOffsetPt (4, -3, 0), 1, axisVec, wheelFill, wheelStroke, 0.1),
    new Wheel (this.centre.newOffsetPt (-4, 3, 0), 1, axisVec, wheelFill, wheelStroke, 0.1),
    new Wheel (this.centre.newOffsetPt (-4, -3, 0), 1, axisVec, wheelFill, wheelStroke, 0.1)
  ];
  this.floor = new Polygon (centreX, centreY, 
    [{x:centreX + 4, y:centreY + 2.9, z:0},
     {x:centreX + 4, y:centreY - 2.9, z:0},
     {x:centreX - 4, y:centreY + 2.9, z:0},
     {x:centreX - 4, y:centreY - 2.9, z:0}]);
  this.floor.translate (0, 0, 1);
  this.halfheight = 0;
  this.background = false;
  this.movable = false;
  this.freeFall = false;
  this.velocity = new Vector3D (0, 0, 0);
  this.angular = new Vector3D (0, 0, 0);
}

Rover.prototype.setAngular = function (ax, ay, az) 
{ this.angular = new Vector3D (ax, ay, az); 
  this.movable = (ax !== 0 || ay !== 0 || az !== 0);
};

Rover.prototype.setColour = function (r, g, b) { this.floor.setColour (r, g, b); };
Rover.prototype.setStroke = function (r, g, b) { this.floor.setStroke (r, g, b); };
Rover.prototype.setStroking = function (bool) { this.stroking = bool; };
Rover.prototype.setSolid = function () {};
Rover.prototype.setFreeFall = function (bool) { this.freeFall = bool; };

Rover.prototype.setVelocity = function (dx, dy, dz) 
{ this.velocity = new Vector3D (dx, dy, dz);
  for (var i = 0; i < this.wheels.length; i++) { this.wheels [i].setVelocity (dx, dy, dz); }
  this.floor.setVelocity (dx, dy, dz);
  this.movable = (dx !== 0 || dy !== 0 || dz !== 0);
};

Rover.prototype.view = function (me, fogNo) {};
// Instead add the components to world:

Rover.prototype.addToWorld = function ()
{ for (var i = 0; i < this.wheels.length; i++) world.add (this.wheels [i]);
  world.add (this.floor);
};

Rover.prototype.explode = function () {};
Rover.prototype.rotateX = function (degs) {};
Rover.prototype.rotateY = function (degs) {};

Rover.prototype.rotateZ = function (degs) { /************** TO DO ****************/ };

Rover.prototype.translate = function (dx, dy, dz) 
{ for (var i = 0; i < this.wheels.length; i++) this.wheels [i].translate (dx, dy, dz);
  this.floor.translate (dx, dy, dz);
};