// Copyright (c) Graham Relf, UK, 2020
// www.grelf.net
'use strict';

function onTeleKeydown (e)
{ var key = e.key.toUpperCase ();
  if (-1 !== 'ABCDEFGHIJKLMNOPQRSTUVWXYZ #,.-'.indexOf (key)) world.teleText += key;
  else if ('BACKSPACE' === key) world.teleText = world.teleText.substring (0, world.teleText.length - 1);
  if ('ENTER' === key) teleport ();
  else teleInput ();
}

function onTeleClick (e)
{ if (!world.keyBounds) return;
  var rect = world.canvas.getBoundingClientRect ();
  var mouseX = e.clientX - rect.left, mouseY = e.clientY - rect.top;
  for (var i = 0; i < world.keyBounds.length; i++)
  { var kbi = world.keyBounds [i];
    if (mouseX >= kbi.xL && mouseX <= kbi.xR && mouseY >= kbi.yT && mouseY <= kbi.yB)
    { switch (kbi.id)
      {
case '<': 
        world.teleText = world.teleText.substring (0, world.teleText.length - 1);
        teleInput ();
        break;
case '>': teleport (); break;
default:
        world.teleText += kbi.id;
        teleInput ();
} } } }

function teleStart (slab)
{ world.uiLocked = true;
  world.animPaused = true;
  world.gate = new Gate ((Math.random () > 0.5) ? 'v' : 'h');
  var route = slab.centre.difference (me.pt);
  var nSteps = 150; // 3s at 50Hz
  world.animData = 
    {dx:route.dx / nSteps, dy:route.dy / nSteps, dz:route.dz / nSteps, nSteps:nSteps};
  document.getElementById ("buttons").innerHTML = "&nbsp; Just enjoy the ride...";
  hint ('');
  moveToGate ();
}

function moveToGate ()
{ var wad = world.animData;
  me.pt.x += wad.dx;
  me.pt.y += wad.dy;
  me.pt.z += wad.dz;
  world.draw ();
  wad.nSteps--;
  if (wad.nSteps > 0) setTimeout (moveToGate, 20);
  else
  { document.getElementById ("info").innerHTML = "";
    setTimeout (drawGate, world.gate.MS_WAIT);
} }

function Gate (vOrH)
{ this.vertical = vOrH.toLowerCase ().startsWith ('v');
  this.rgb = [200, 200, 200];
  this.rgbUp = [false, false, false];
  this.styles = [];
  for (var i = 0; i < 16; i++) this.styles.push (this.modRGB ());
  this.count = random (100, 200);
  this.MS_WAIT = 80;
  this.wd = world.screenWd; this.wd1 = this.wd / 4; this.wd2 = this.wd / 2; this.wd3 = this.wd1 + this.wd2;
  this.ht = world.screenHt; this.ht1 = this.ht / 4; this.ht2 = this.ht / 2; this.ht3 = this.ht1 + this.ht2;
  this.teledoor = world.images ['teledoor'];
  this.teleframe = world.images ['teleframe'];
}

Gate.prototype.modRGB = function ()
{ for (var i = 0; i < 3; i++)
  { var dd = (Math.random () * PI10000) & 0x17;
    if (this.rgbUp [i])
    { if (this.rgb [i] + dd < 256) this.rgb [i] += dd; 
      else { this.rgb [i] -= dd; this.rgbUp [i] = false; }
    }
    else
    { if (this.rgb [i] - dd >= 0) this.rgb [i] -= dd; 
      else { this.rgb [i] += dd; this.rgbUp [i] = true; }
  } }
  return makeCssColour (this.rgb [0], this.rgb [1], this.rgb [2]);
};

function drawGate ()
{ var wg = world.gate, g2 = world.g2, xg, yg;
  var x0 = wg.wd2 - wg.teleframe.width / 2;
  var x1 = wg.wd2 - wg.teledoor.width / 2;
  var y0 = wg.ht2 - wg.teleframe.height / 2;
  var y1 = wg.ht2 - wg.teledoor.height / 2 + 26;
  if (wg.vertical) { xg = wg.wd; yg = 0; }
  else { xg = 0; yg = wg.ht; }
  var gradient = g2.createLinearGradient (0, 0, xg, yg);
  var len = wg.styles.length, dx = 0.5 / len;
  for (var i = 0; i < len; i++)
  { var idx = i * dx, fmsi = wg.styles [i];
    gradient.addColorStop (0.5 + idx, fmsi);
    gradient.addColorStop (0.5 - idx, fmsi);
  }
  for (i = len - 1; i >= 1; i--) wg.styles [i] = wg.styles [i - 1];
  wg.styles [0] = wg.modRGB ();
  g2.fillStyle = gradient;
  g2.fillRect (0, 0, wg.wd, wg.ht);
  g2.strokeStyle = '#000';
  var lw = (wg.vertical ? wg.wd : wg.ht) / wg.count;
  g2.lineWidth = lw;
  g2.fillStyle = '#000';
  if (wg.vertical)
  { fillTriangle (g2, 0, 0, wg.wd, 0, wg.wd2, wg.ht1);
    fillTriangle (g2, 0, wg.ht, wg.wd, wg.ht, wg.wd2, wg.ht3);
    strokeLine (g2, wg.wd2, 0, wg.wd2, wg.ht);
  }
  else
  { fillTriangle (g2, 0, 0, 0, wg.ht, wg.wd1, wg.ht2);
    fillTriangle (g2, wg.wd, 0, wg.wd, wg.ht, wg.wd3, wg.ht2);  
    strokeLine (g2, 0, wg.ht2, wg.wd, wg.ht2);
  }
  if (lw > 4)
  { if (lw >= wg.teleframe.width)
    { g2.drawImage (wg.teleframe, x0, y0);
      g2.drawImage (wg.teledoor, x1, y1);
    }
    else
    { var scale = lw / wg.teleframe.width, scaleHt = scale * wg.teleframe.height;
      g2.drawImage (wg.teleframe, wg.wd2 - lw / 2, wg.ht2 - scaleHt / 2, lw, scaleHt);
      g2.drawImage (wg.teledoor, wg.wd2 - lw / 2, wg.ht2 - scaleHt / 2, lw, scaleHt);
  } }
  if (wg.count > 0) { wg.count--; setTimeout (drawGate, wg.MS_WAIT); }
  else
  { g2.lineWidth = 1;
    g2.fillStyle = '#000';
    g2.fillRect (0, 0, wg.wd, wg.ht);
    g2.drawImage (wg.teleframe, x0, y0);
    g2.drawImage (wg.teledoor, x1, y1);
    document.getElementById ("hint").innerHTML = "";
    world.animData = {x1:x1, y1:y1, nSteps:10, door:wg.teledoor, frame:wg.teleframe};
    setTimeout (openTele, 2000);
} }

function openTele ()
{ var wad = world.animData, g2 = world.g2;
  var wd = wad.door.width, ht = wad.door.height;
  g2.fillStyle = '#000';
  g2.fillRect (wad.x1, wad.y1, wd, ht);
  wad.nSteps--;
  var wdFrac = wd * wad.nSteps / 10;
  if (wad.nSteps > 0) 
  { g2.drawImage (wad.door, wad.x1 + wd - wdFrac, wad.y1, wdFrac, ht);
    setTimeout (openTele, 250);
  }
  else
  { wad.nSteps = 20;
    setTimeout (enterTele, 250);
} }

function enterTele ()
{var wad = world.animData, g2 = world.g2;
  var wd = wad.frame.width, ht = wad.frame.height;
  g2.fillStyle = '#000';
  g2.fillRect (0, 0, world.screenWd, world.screenHt);
  wad.nSteps--;
  if (wad.nSteps > 0) 
  { var scale = 20 / wad.nSteps;
    g2.drawImage (wad.frame, world.screenWd2 - scale * wd / 2, world.screenHt2 - scale * ht / 2,
      scale * wd, scale * ht);
    setTimeout (enterTele, 50);
  }
  else showTelePanel ();
}

function showTelePanel ()
{ var g2 = world.g2, wd2 = world.screenWd2;
  g2.font = "bold 24px sans-serif";
  g2.fillStyle = '#fff';
  var s = "T E L E P O R T E";
  var sw2 = g2.measureText (s).width / 2;
  g2.fillText (s, wd2 - sw2, 100);
  s = "Where would you like to go?";
  sw2 = g2.measureText (s).width / 2;
  g2.fillText (s, wd2 - sw2, 150);
  g2.fillRect (wd2 - 175, 180, 350, 32);
  world.keyboard = new Keyboard (world.screenWd * 0.2, world.screenHt * 0.4, world.screenWd * 0.6);
  world.keyBounds = world.keyboard.draw ('');
  world.inTele = true;
  world.teleText = '';
  document.getElementById ("buttons").innerHTML = '';
  hint ("The codes for the keys you type will be<br>converted to an (x, y) position (a hash code)")
  world.canvas.removeEventListener ('click', onClick);
  window.removeEventListener ('keydown', onKeydown, true);
  world.canvas.addEventListener ('click', onTeleClick);
  window.addEventListener ('keydown', onTeleKeydown, true);
  world.uiLocked = false;
}

function teleInput ()
{ var g2 = world.g2, wd2 = world.screenWd2;
  g2.fillStyle = '#fff';
  g2.fillRect (wd2 - 175, 180, 350, 32);
  g2.font = "24px sans-serif";
  g2.fillStyle = '#000';
  g2.fillText (world.teleText, wd2 - 170, 200);
}

function teleport ()
{ var s = world.teleText;
  var f = Math.PI * 0xffffffff;
  var xy = PI10000;
  for (var i = 0; i < s.length; i++) { xy += s.charCodeAt (i) * f; }
  me.pt.x = ((xy & 0x7fff0000) >> 16) % (2 * RANGE) - RANGE;
  me.pt.y = (xy & 0x00007fff) % (2 * RANGE) - RANGE;
  me.b = 0;
  hint (s);
  world.inTele = false;
  world.canvas.removeEventListener ('click', onTeleClick);
  window.removeEventListener ('keydown', onTeleKeydown, true);
  world.canvas.addEventListener ('click', onClick);
  window.addEventListener ('keydown', onKeydown, true);
  world.uiLocked = false;
  world.animPaused = false;
  document.getElementById ("view").focus ();
  showButtons ();
  animate ();
}

function Keyboard (xLeft, yTop, width)
{ this.xLeft = xLeft;
  this.yTop = yTop;
  this.width = width;
  this.rows = ['1234567890-<', 'QWERTYUIOP>', 'ASDFGHJKL#', 'ZXCVBNM,.', ' '];
  this.maxRowLen = 0;
  for (var i = 0; i < this.rows.length; i++)
  { if (this.rows [i].length > this.maxRowLen) this.maxRowLen = this.rows [i].length; }
}

Keyboard.prototype.redraw = function (id)
{ this.draw (this.xLeft, this.yTop, this.width, id);  };

/* id is a key to light up, '' if none */
Keyboard.prototype.draw = function (id)
{ var keyBounds = [];
  var g2 = world.g2;
  var fontPx = Math.round (24 * this.width / 800);
  g2.font = "bold " + fontPx + "px sans-serif";
  var nRows = this.rows.length;
  var height = 0.8 * nRows * this.width / this.maxRowLen;
  var yCentre = this.yTop + height / 2;
  var xCentre = this.xLeft + this.width / 2;
  var lw = 4, r = 3;
  drawRoundedRect (g2, this.xLeft, this.yTop, this.width, height, lw, r, '#555', '#ccc');
  var keyPitch = 0.8 * this.width / (this.maxRowLen);
  var keySize = keyPitch * 0.8;
  var keySize2 = keySize / 2;
  var yRow0 = yCentre - keyPitch * (nRows / 2 - 0.5) ;
  for (var iy = 0; iy < nRows; iy++)
  { var rowLen = this.rows [iy].length;
    var yKeyCentre = yRow0 + iy * keyPitch;
    var x0 = xCentre - keyPitch * (rowLen / 2 - 0.5);
    for (var ix = 0; ix < rowLen; ix++)
    { var s = this.rows [iy].charAt (ix);
      var sw2 = g2.measureText (s).width / 2;
      var xKeyCentre = x0 + ix * keyPitch;
      var xL = xKeyCentre - keySize2, yT = yKeyCentre - keySize2;
      switch (s)
      {
case ' ':
        xL -= 2 * keySize;
        drawRoundedRect (g2, xL, yT, keySize * 5 - lw, keySize - lw, lw, r, '#000', '#fff');
        keyBounds.push ({xL: xL, xR: xL + keySize * 5, yT: yT, yB: yT + keySize, id:s});
        break;
case '<':
        drawRoundedRect (g2, xL, yT, keySize * 2 - lw, keySize - lw, lw, r, '#000', '#fff');
        g2.fillStyle = (id === s) ? '#fa0' : '#000';
        g2.fillText ('< x', xKeyCentre - sw2 - r, yKeyCentre + r);
        keyBounds.push ({xL: xL, xR: xL + keySize * 2, yT: yT, yB: yT + keySize, id:s});
        break;
case '>':
        drawRoundedRect (g2, xL, yT, keySize * 2 - lw, keySize - lw, lw, r, '#000', '#fff');
        g2.fillStyle = (id === s) ? '#fa0' : '#000';
        g2.fillText ('Enter', xKeyCentre - sw2 - r, yKeyCentre + r);
        keyBounds.push ({xL: xL, xR: xL + keySize * 2, yT: yT, yB: yT + keySize, id:s});
        break;
default:
        drawRoundedRect (g2, xL, yT, keySize - lw, keySize - lw, lw, r, '#000', '#fff');
        g2.fillStyle = (id === s) ? '#fa0' : '#000';
        g2.fillText (s, xKeyCentre - sw2 - r, yKeyCentre + r);
        keyBounds.push ({xL: xL, xR: xL + keySize, yT: yT, yB: yT + keySize, id:s});
  } } }
  g2.lineWidth = 1;
  g2.font = "bold 24px sans-serif";
  return keyBounds;
};

function drawRoundedRect (g2, xLeft, yTop, width, height, r, lineWd, sStyle, fStyle)
{ var L2 = Math.floor (lineWd / 2);
  var xOutL = xLeft + L2, xOutR = xLeft + width - L2, yOutT = yTop + L2, yOutB = yTop + height - L2;
  var xInL = xOutL + r, xInR = xOutR - r, yInT = yOutT + r, yInB = yOutB - r;
  g2.lineWidth = lineWd;
  g2.strokeStyle = sStyle;
  g2.beginPath ();
  g2.moveTo (xInL, yOutT);
  g2.lineTo (xInR, yOutT);
  g2.arc (xInR, yInT, r, 3 * PI_BY_2, 0);
  g2.lineTo (xOutR, yInB);
  g2.arc (xInR, yInB, r, 0, PI_BY_2);
  g2.lineTo (xInL, yOutB);
  g2.arc (xInL, yInB, r, PI_BY_2, Math.PI);
  g2.lineTo(xOutL, yInT);
  g2.arc (xInL, yInT, r, Math.PI, 3 * PI_BY_2);
  g2.closePath ();
  if (fStyle !== '')
  { g2.fillStyle = fStyle;
    g2.fill ();
  }
  g2.stroke ();
}