// Copyright (c) Graham Relf, UK, 2021
// www.grelf.net
'use strict';

// Spiral around vertical (z)
function Coil (centre, radius, height, nTurns, sectionsPerTurn, colour)
{ var dzTurn = height / nTurns;
  var dRadSection = 2 * Math.PI / sectionsPerTurn;
  var dzSection = dzTurn / sectionsPerTurn;
  var nVerts = nTurns * sectionsPerTurn;
  var vertices = [];
  var ht2 = height / 2;
  var z = centre.z - ht2;
  for (var i = 0, aRad = 0; i < nVerts; i++, z += dzSection)
  { vertices.push (
      new Point3D (centre.x + radius * Math.sin (aRad), centre.y + radius * Math.cos (aRad), z));
    aRad += dRadSection;
  }
  Wire.call (this, vertices, colour);
  this.centre = centre;
  this.radius = radius;
  this.height = height;
  this.halfheight = ht2;
  this.attachedShapes = [];
}

Coil.prototype = Object.create (Wire.prototype);
Coil.prototype.constructor = Coil;

Coil.prototype.attach = function (shape) { this.attachedShapes.push (shape); };

// Compress (factor < 1) or stretch (> 1)
Coil.prototype.spring = function (factor)
{ var vs = this.vertices;
  var z = vs [0].z, dz = (vs [1].z - z) * factor;
  for (var i = 0; i < vs.length; i++, z += dz) vs [i].z = z;
  var oldHt = this.height;
  this.height *= factor;
  this.halfheight *= factor;
  this.centre.z = vs [0].z + this.halfheight;
  var ats = this.attachedShapes;
  for (var i = 0; i < ats.length; i++) ats [i].translate (0, 0, this.height - oldHt);
};
