// Copyright (c) Graham Relf, UK, 2020
// www.grelf.net
'use strict';

function Cuboid (xCentre, yCentre, zCentre, halfwidth, halfdepth, halfheight)
{ Shape.call (this);
  this.centre = new Point3D (xCentre, yCentre, zCentre);
  this.radius = Math.sqrt (halfwidth * halfwidth + halfdepth * halfdepth + halfheight * halfheight);
  this.fillColours = // default:
  [ {r:96, g:96, b:96}, // 0: grey top #666
    {r:128, g:128, b:128}, // 1: grey bottom #888
    {r:176, g:144, b:144}, // 2: red north #b99
    {r:176, g:176, b:144}, // 3: yellow south #bb9
    {r:144, g:176, b:144}, // 4: green east #9b9
    {r:144, g:160, b:176}  // 5: blue west #9ab
  ];
  if (arguments.length > 6) this.fillColours = arguments [6]; 
  this.vertices [0] = new Point3D (xCentre + halfwidth, yCentre + halfdepth, zCentre + halfheight);
  this.vertices [1] = new Point3D (xCentre - halfwidth, yCentre + halfdepth, zCentre + halfheight);
  this.vertices [2] = new Point3D (xCentre + halfwidth, yCentre - halfdepth, zCentre + halfheight);
  this.vertices [3] = new Point3D (xCentre - halfwidth, yCentre - halfdepth, zCentre + halfheight);
  this.vertices [4] = new Point3D (xCentre + halfwidth, yCentre + halfdepth, zCentre - halfheight);
  this.vertices [5] = new Point3D (xCentre - halfwidth, yCentre + halfdepth, zCentre - halfheight);
  this.vertices [6] = new Point3D (xCentre + halfwidth, yCentre - halfdepth, zCentre - halfheight);
  this.vertices [7] = new Point3D (xCentre - halfwidth, yCentre - halfdepth, zCentre - halfheight);
  this.faces [0] = new Face (this, this.centre.newOffsetPt (0, 0, halfheight),
    [this.vertices [0], this.vertices [1], this.vertices [3], this.vertices [2]], this.fillColours [0]); 
  this.faces [1] = new Face (this, this.centre.newOffsetPt (0, 0, -halfheight),
    [this.vertices [4], this.vertices [5], this.vertices [7], this.vertices [6]], this.fillColours [1]);
  this.faces [2] = new Face (this, this.centre.newOffsetPt (0, halfdepth, 0),
    [this.vertices [0], this.vertices [1], this.vertices [5], this.vertices [4]], this.fillColours [2]);
  this.faces [3] = new Face (this, this.centre.newOffsetPt (0, -halfdepth, 0),
    [this.vertices [2], this.vertices [3], this.vertices [7], this.vertices [6]], this.fillColours [3]);
  this.faces [4] = new Face (this, this.centre.newOffsetPt (halfwidth, 0, 0),
    [this.vertices [0], this.vertices [2], this.vertices [6], this.vertices [4]], this.fillColours [4]); 
  this.faces [5] = new Face (this, this.centre.newOffsetPt (-halfwidth, 0, 0),
    [this.vertices [1], this.vertices [3], this.vertices [7], this.vertices [5]], this.fillColours [5]);
  this.halfwidth = halfwidth;
  this.halfdepth = halfdepth;
  this.halfheight = halfheight;
  this.background = false;
  this.freeFall = false;
  this.velocity = new Vector3D (0, 0, 0);
  this.angular = new Vector3D (0, 0, 0);
}

Cuboid.prototype = Object.create (Shape.prototype);
Cuboid.prototype.constructor = Cuboid;
