// Copyright (c) Graham Relf, UK, 2020
// www.grelf.net
'use strict';
const GR = (1 + Math.sqrt (5)) * 0.5; // Golden ratio

function Dodecahedron (xCentre, yCentre, zCentre, radius)
{ Shape.call (this);
  this.centre = new Point3D (xCentre, yCentre, zCentre);
  this.radius = radius;
  var unit = radius / Math.sqrt (3), gr = unit * GR, recipGR = unit / GR;
  this.fillColours = // default:
  [ {r:96, g:96, b:96}, // grey bottom #666
    {r:128, g:128, b:128}, // grey top #888
    {r:176, g:144, b:144}, // red north #b99
    {r:176, g:176, b:144}, // yellow south #bb9
    {r:144, g:176, b:144}, // green east #9b9
    {r:144, g:160, b:176},  // blue west #9ab
    {r:96, g:96, b:96}, // grey bottom #666
    {r:128, g:128, b:128}, // grey top #888
    {r:176, g:144, b:144}, // red north #b99
    {r:176, g:176, b:144}, // yellow south #bb9
    {r:144, g:176, b:144}, // green east #9b9
    {r:144, g:160, b:176}  // blue west #9ab
  ];
  if (arguments.length > 4) this.fillColours = arguments [4]; 
  this.vertices = new Array (20);
  this.vertices [0] = new Point3D (xCentre + unit, yCentre + unit, zCentre + unit);
  this.vertices [1] = new Point3D (xCentre + gr, yCentre + recipGR, zCentre);
  this.vertices [2] = new Point3D (xCentre + gr, yCentre - recipGR, zCentre);
  this.vertices [3] = new Point3D (xCentre + unit, yCentre - unit, zCentre + unit);
  this.vertices [4] = new Point3D (xCentre + recipGR, yCentre, zCentre + gr);
  this.vertices [5] = new Point3D (xCentre - recipGR, yCentre, zCentre + gr);
  this.vertices [6] = new Point3D (xCentre - unit, yCentre + unit, zCentre + unit);
  this.vertices [7] = new Point3D (xCentre, yCentre + gr, zCentre + recipGR);
  this.vertices [8] = new Point3D (xCentre, yCentre + gr, zCentre - recipGR);
  this.vertices [9] = new Point3D (xCentre - unit, yCentre + unit, zCentre - unit);
  this.vertices [10] = new Point3D (xCentre - gr, yCentre + recipGR, zCentre);
  this.vertices [11] = new Point3D (xCentre - gr, yCentre - recipGR, zCentre);
  this.vertices [12] = new Point3D (xCentre - unit, yCentre - unit, zCentre + unit);
  this.vertices [13] = new Point3D (xCentre, yCentre - gr, zCentre + recipGR);
  this.vertices [14] = new Point3D (xCentre, yCentre - gr, zCentre - recipGR);
  this.vertices [15] = new Point3D (xCentre - unit, yCentre - unit, zCentre - unit);
  this.vertices [16] = new Point3D (xCentre - recipGR, yCentre, zCentre - gr);
  this.vertices [17] = new Point3D (xCentre + recipGR, yCentre, zCentre - gr);
  this.vertices [18] = new Point3D (xCentre + unit, yCentre + unit, zCentre - unit);
  this.vertices [19] = new Point3D (xCentre + unit, yCentre - unit, zCentre - unit);
  this.faces = new Array (12);
  this.makeFace (0, [0, 1, 2, 3, 4]);
  this.makeFace (1, [1, 0, 7, 8, 18]);
  this.makeFace (2, [2, 1, 18, 17, 19]);
  this.makeFace (3, [3, 2, 19, 14, 13]);
  this.makeFace (4, [4, 3, 13, 12, 5]);
  this.makeFace (5, [5, 6, 7, 0, 4]);
  this.makeFace (6, [6, 5, 12, 11, 10]);
  this.makeFace (7, [7, 6, 10, 9, 8]);
  this.makeFace (8, [8, 9, 16, 17, 18]);
  this.makeFace (9, [9, 10, 11, 15, 16]);
  this.makeFace (10, [14, 19, 17, 16, 15]);
  this.makeFace (11, [11, 12, 13, 14, 15]);
  this.halfheight = radius;
  this.background = false;
  this.freeFall = false;
  this.velocity = new Vector3D (0, 0, 0);
  this.angular = new Vector3D (0, 0, 0);
}

Dodecahedron.prototype = Object.create (Shape.prototype);
Dodecahedron.prototype.constructor = Dodecahedron;
