// Copyright (c) Graham Relf, UK, 2020
// www.grelf.net
'use strict';

// Doors are rectangles; 2 vertices are hinge points (3D) and a 3D vector points to the opposite side.
// Hinges must be parallel to a coordinate axis.
// Doors are created in closed position (angle = 0).
function Door (hingePt1, hingePt2, vector, clockwise)
{ Shape.call (this);
  this.fillColours = [{r:96, g:96, b:96}]; // grey #666
  if (arguments.length > 4) this.fillColours = arguments [4];
  this.vertices.push (hingePt1);
  this.vertices.push (hingePt2);
  this.vertices.push (hingePt2.newOffsetPt (vector.dx, vector.dy, vector.dz));
  this.vertices.push (hingePt1.newOffsetPt (vector.dx, vector.dy, vector.dz));
  this.centre = new Point3D ((hingePt1.x + hingePt2.x) / 2, (hingePt1.y + hingePt2.y) / 2,
    (hingePt1.z + hingePt2.z) / 2); // of rotation: mid-hinge
  this.faces [0] = new Face (this, this.centre.clone (), this.vertices, this.fillColours [0]);
  this.halfheight = 0; // Not needed for a door <= cannot move off ground
  this.background = false;
  this.movable = false;
  this.freeFall = false;
  this.velocity = new Vector3D (0, 0, 0);
  this.angular = new Vector3D (0, 0, 0);
  this.clockwise = clockwise;
  this.closed = true;
  if (hingePt1.x !== hingePt2.x) this.axis = 'x';
  else if (hingePt1.y !== hingePt2.y) this.axis = 'y';
  else if (hingePt1.z !== hingePt2.z) this.axis = 'z';
  this.isTarget = true;
  world.doors.push (this);
}

Door.prototype = Object.create (Shape.prototype);
Door.prototype.constructor = Door;

Door.prototype.tooClose = function (pt3d)
{ if (pt3d.distance (this.vertices [0]) < world.CLOSEST * 2
      && Math.abs (this.faces [0].getPoint3DDistance (pt3d)) < world.CLOSEST) return true;
  return false;
};

Door.prototype.response = function ()
{ if (this.closed) this.open (); else this.close (); };

Door.prototype.open = function ()
{ world.uiLocked = true;
  world.animData = { door:this, opening:true, nSteps:10, dDegs: 9 };
  doorOpenOrClose ();
};

Door.prototype.close = function ()
{ world.uiLocked = true;
  world.animData = { door:this, opening:false, nSteps:10, dDegs: 9 };
  doorOpenOrClose ();
};

function doorOpenOrClose ()
{ var wad = world.animData;
  var dr = wad.door;
  var dDegs = (dr.clockwise) ? wad.dDegs : -wad.dDegs;
  if (!dr.closed) dDegs = - dDegs;
  switch (dr.axis)
  { case 'x': dr.rotateX (dDegs); break;
    case 'y': dr.rotateY (dDegs); break;
    case 'z': dr.rotateZ (dDegs); break;
  }
  world.draw ();
  wad.nSteps--;
  if (wad.nSteps > 0) setTimeout (doorOpenOrClose, 20);
  else
  { dr.closed = wad.opening ? false : true;
    hint ("");
    world.uiLocked = false;
    animate ();
  }
}