// Copyright (c) Graham Relf, UK, 2020
// www.grelf.net
'use strict';
//const GR = (1 + Math.sqrt (5)) * 0.5; // Golden ratio - DEFINED IN dodecahedron.js

function Icosahedron (xCentre, yCentre, zCentre, radius)
{ Shape.call (this);
  this.centre = new Point3D (xCentre, yCentre, zCentre);
  this.radius = radius;
  var unit = radius / Math.sqrt (3), gr = unit * GR;
  this.fillColours = // default:
  [ {r:96, g:96, b:96}, // grey bottom #666
    {r:128, g:128, b:128}, // grey top #888
    {r:176, g:144, b:144}, // red north #b99
    {r:176, g:176, b:144}, // yellow south #bb9
    {r:144, g:176, b:144}, // green east #9b9
    {r:144, g:160, b:176},  // blue west #9ab
    {r:96, g:96, b:96}, // grey bottom #666
    {r:128, g:128, b:128}, // grey top #888
    {r:176, g:144, b:144}, // red north #b99
    {r:176, g:176, b:144}, // yellow south #bb9
    {r:144, g:176, b:144}, // green east #9b9
    {r:144, g:160, b:176},  // blue west #9ab
    {r:96, g:96, b:96}, // grey bottom #666
    {r:128, g:128, b:128}, // grey top #888
    {r:176, g:144, b:144}, // red north #b99
    {r:176, g:176, b:144}, // yellow south #bb9
    {r:144, g:176, b:144}, // green east #9b9
    {r:144, g:160, b:176},  // blue west #9ab
    {r:144, g:176, b:144}, // green east #9b9
    {r:144, g:160, b:176}  // blue west #9ab
  ];
  if (arguments.length > 4) this.fillColours = arguments [4]; 
  this.vertices = new Array (12);
  this.vertices [0] = new Point3D (xCentre, yCentre + unit, zCentre + gr);
  this.vertices [1] = new Point3D (xCentre, yCentre - unit, zCentre + gr);
  this.vertices [2] = new Point3D (xCentre + gr, yCentre, zCentre + unit);
  this.vertices [3] = new Point3D (xCentre - gr, yCentre, zCentre + unit);
  this.vertices [4] = new Point3D (xCentre + unit, yCentre + gr, zCentre);
  this.vertices [5] = new Point3D (xCentre + unit, yCentre - gr, zCentre);
  this.vertices [6] = new Point3D (xCentre - unit, yCentre - gr, zCentre);
  this.vertices [7] = new Point3D (xCentre - unit, yCentre + gr, zCentre);
  this.vertices [8] = new Point3D (xCentre + gr, yCentre, zCentre - unit);
  this.vertices [9] = new Point3D (xCentre - gr, yCentre, zCentre - unit);
  this.vertices [10] = new Point3D (xCentre, yCentre - unit, zCentre - gr);
  this.vertices [11] = new Point3D (xCentre, yCentre + unit, zCentre - gr);
  this.faces = new Array (20);
  this.makeFace (0, [0, 1, 2]);
  this.makeFace (1, [3, 6, 9]);//
  this.makeFace (2, [3, 0, 1]);
  this.makeFace (3, [0, 7, 3]);
  this.makeFace (4, [0, 2, 4]);
  this.makeFace (5, [2, 4, 8]);
  this.makeFace (6, [5, 1, 2]);
  this.makeFace (7, [5, 1, 6]);
  this.makeFace (8, [3, 1, 6]);
  this.makeFace (9, [3, 9, 7]);
  this.makeFace (10, [0, 4, 7]);
  this.makeFace (11, [4, 7, 11]);
  this.makeFace (12, [4, 8, 11]);
  this.makeFace (13, [7, 9, 11]);
  this.makeFace (14, [2, 5, 8]);
  this.makeFace (15, [5, 8, 10]);
  this.makeFace (16, [5, 6, 10]);
  this.makeFace (17, [6, 9, 10]);
  this.makeFace (18, [9, 10, 11]);
  this.makeFace (19, [8, 10, 11]);
  this.halfheight = radius;
  this.background = false;
  this.freeFall = false;
  this.velocity = new Vector3D (0, 0, 0);
  this.angular = new Vector3D (0, 0, 0);
}

Icosahedron.prototype = Object.create (Shape.prototype);
Icosahedron.prototype.constructor = Icosahedron;
