// Copyright (c) Graham Relf, UK, 2020
// www.grelf.net
'use strict';

const SPRITE = false, FACE = true; //for loadImage()

function loadImage (name, filePath, forFaces)
{ world.nImsToLoad++;
  var im = new Image ();
  im.loaded = false;
  im.onerror = function ()
  { error ('ERROR: Failed to load ' + filePath); };
  im.onload = function ()
  { im.foggy = new Array (8);
    im.foggy [0] = im;
    im.forFaces = forFaces;
    if (forFaces)
    { im.corners = // Clockwise from top left
[{x:0, y:0}, {x:im.width - 1, y:0}, {x:im.width - 1, y:im.height - 1}, {x:0, y:im.height - 1}];
      var cnv = document.createElement ('canvas');
      cnv.width = im.width;
      cnv.height = im.height;
      var g2 = cnv.getContext ('2d');
      g2.drawImage (im, 0, 0);
      im.foggy [0].data = g2.getImageData (0, 0, im.width, im.height);
      if (world.withFog) fogAll (im); // 1..7
    }
    world.images [name] = im;
    im.loaded = true;
    world.nImsLoaded++;
  };
  im.src = filePath;
}

function createFoggedImageAndData (im, fogNo) // 0..1
{ var cnv = document.createElement ('canvas');
  cnv.width = im.width;
  cnv.height = im.height;
  var g2 = cnv.getContext ('2d');
  g2.drawImage (im, 0, 0); // Full size
  var imData = g2.getImageData (0, 0, im.width, im.height);
  var px = imData.data;
  for (var i = 0; i < px.length; i++)
  { var i3 = i + 3;
    if (0 === px [i3]) i = i3; // skip transparent px
    else
    { px [i] += (world.sky.r - px [i]) * fogNo; i++;
      px [i] += (world.sky.g - px [i]) * fogNo; i++;
      px [i] += (world.sky.b - px [i]) * fogNo; i++;
  } } // loop inc skips alpha
  g2.putImageData (imData, 0, 0);
  var fogged = new Image ();
  fogged.onload = function () { fogged.loaded = true; }
  fogged.src = cnv.toDataURL ('image/png'); // Get result as an Image
  return {im:fogged, data:imData};
}

function fogAll (im) // 1..7
{ for (var iFogNo = 1; iFogNo < 8; iFogNo++)
  { var fogged = createFoggedImageAndData (im, iFogNo / 7);
    im.foggy [iFogNo] = fogged.im;
    if (im.forFaces)
    { im.foggy [iFogNo].data = fogged.data;
      im.foggy [iFogNo].corners = im.corners;
} } }

function fog (r, g, b, fogNo)// fogNo = 0..1
{ r += (world.sky.r - r) * fogNo;
  g += (world.sky.g - g) * fogNo;
  b += (world.sky.b - b) * fogNo;
  return makeCssColour (Math.floor (r), Math.floor (g), Math.floor (b));  
}

function makeCssColour (r, g, b) // each 0..255
{ var rs = r.toString (16), gs = g.toString (16), bs = b.toString (16);
  if (r < 16) rs = '0' + rs;
  if (g < 16) gs = '0' + gs;
  if (b < 16) bs = '0' + bs;
  return '#' + rs + gs + bs; 
}

/* Returns an array with 4 elements: RGBA. */
ImageData.prototype.getPixel = function (x, y)
{ var i = (this.width * Math.round (y) + Math.round (x)) * 4;
  // NB: JavaScript will try to index arrays with non-integers!
  return [this.data [i], this.data [i + 1], this.data [i + 2], this.data [i + 3]];
};

/* rgba is an array with 4 elements: RGBA. Ignores transparent pixels (A = 0). */
ImageData.prototype.setPixel = function (x, y, rgba)
{ if (0 === rgba [3]) return;
  var i = (this.width * Math.round (y) + Math.round (x)) * 4;
  this.data [i] = rgba [0];
  this.data [i + 1] = rgba [1];
  this.data [i + 2] = rgba [2];
  this.data [i + 3] = rgba [3];
};

ImageData.prototype.getRow = function (x, y, wd)
{ var srcI = (this.width * Math.round (y) + Math.round (x)) * 4;
  var row = new Uint8ClampedArray (wd * 4);
  for (var dstI = 0; dstI < wd * 4; srcI++, dstI++) { row [dstI] = this.data [srcI]; }
  return row;
};

ImageData.prototype.setRow = function (x, y, row)
{ var dstI = (this.width * Math.round (y) + Math.round (x)) * 4;
  for (var srcI = 0; srcI < row.length; srcI++, dstI++) { this.data [dstI] = row [srcI]; }
};

ImageData.prototype.crop = function (x0, y0, wd, ht)
{ var cropped = { width:wd, height:ht, data:new Uint8ClampedArray (wd * ht * 4) }; // new ImageData
  for (var srcY = y0, dstY = 0; dstY < ht; srcY++, dstY++)
  { cropped.setRow (0, dstY, this.data.getRow (x0, srcY, wd)); }
  return cropped;
};