// Copyright (c) Graham Relf, UK, 2020
// www.grelf.net
'use strict';

function Point3D (x, y, z) { this.x = x; this.y = y; this.z = z; }

Point3D.prototype.offset = function (dx, dy, dz) { this.x += dx; this.y += dy; this.z += dz; };

Point3D.prototype.newOffsetPt = function (dx, dy, dz)
{ return new Point3D (this.x + dx, this.y + dy, this.z + dz); };

Point3D.prototype.distance = function (otherPt)
{ var dx = otherPt.x - this.x, dy = otherPt.y - this.y, dz = otherPt.z - this.z;
  return Math.sqrt (dx * dx + dy * dy + dz * dz);
};

Point3D.prototype.distanceAndBearing = function (otherPt)
{ var dx = otherPt.x - this.x, dy = otherPt.y - this.y, dz = otherPt.z - this.z;
  var dx2dy2 = dx * dx + dy * dy;
  var distance = Math.sqrt (dx2dy2 + dz * dz);
  var bearingRad = Math.atan2 (dx, dy); // clockwise from north
  return {d:distance, b:bearingRad};
};

Point3D.prototype.distanceAndAngles = function (otherPt)
{ var dx = otherPt.x - this.x, dy = otherPt.y - this.y, dz = otherPt.z - this.z;
  var dx2dy2 = dx * dx + dy * dy;
  var distance = Math.sqrt (dx2dy2 + dz * dz);
  var bearingRad = Math.atan2 (dx, dy); // clockwise from north
  var altitudeRad = Math.atan2 (dz, Math.sqrt (dx2dy2));
  return {d:distance, b:bearingRad, a:altitudeRad};
};

/* Get vector from otherPt to this */
Point3D.prototype.difference = function (otherPt)
{ return new Vector3D (this.x - otherPt.x, this.y - otherPt.y, this.z - otherPt.z); };

Point3D.prototype.toString = function ()
{ return '(' + this.x.toFixed (1) + ', ' + this.y.toFixed (1) + ', ' + this.z.toFixed (1) + ')'; };

Point3D.prototype.clone = function ()
{ return new Point3D (this.x, this.y, this.z); };
