// Copyright (c) Graham Relf, UK, 2020
// www.grelf.net
'use strict';

// Polygons are usually created in an x-y plane, z = 0. Rotate about x or y to stand them up.

function Polygon (xCentre, yCentre, vertices) // Array of {x,y,z} for vertices. Usually z:0
{ Shape.call (this);
  this.centre = new Point3D (xCentre, yCentre, 0);
  this.fillColours = [{r:96, g:96, b:96}]; // grey #666
  if (arguments.length > 3) this.fillColours = arguments [3];
  var nVerts = vertices.length;
  for (var i = 0; i < nVerts; i++)
  { var vi = vertices [i];
    this.vertices [i] = new Point3D (vi.x, vi.y, vi.z);
  }
  this.faces [0] = new Face (this, this.centre.clone (), this.vertices, this.fillColours [0]);
  this.halfheight = 0;
  this.background = false;
  this.movable = false;
  this.freeFall = false;
  this.velocity = new Vector3D (0, 0, 0);
  this.angular = new Vector3D (0, 0, 0);
}

Polygon.prototype = Object.create (Shape.prototype);
Polygon.prototype.constructor = Polygon;

// Returns a Prism. Must be used when the polygon is still flat in the x-y plane
Polygon.prototype.extrude = function (height)
{ var ht2 = height / 2;
  return new Prism (this.centre.x, this.centre.y, this.centre.z + ht2, ht2, this.vertices);
};

// For exploded fragment landing - assumed triangular
Polygon.prototype.level = function ()
{ // Simple cheat
  var c = this.centre, v = this.vertices;
  var d0 = v [0].distance (v [1]);
  var d1 = v [0].distance (v [2]);
  v [0].z = 0;
  c = v [0];
  v [1] = c.newOffsetPt (d0, 0, 0);
  v [2] = c.newOffsetPt (0, d1, 0);
  this.rotateZ (random (0, 360));
/*
  var normal = this.faces [0].getNormalVector ();
//console.log (normal);
  var inclinationDegs = Math.acos (normal.dz / normal.magnitude ()) * RAD2DEG;
//console.log (inclinationDegs);
  this.rotateZ (Math.atan2 (normal.dy, normal.dx) * RAD2DEG)
  this.rotateY (inclinationDegs);
  this.translate (0, 0, -this.centre.z);
//console.log (this.vertices);
  for (var i = 0; i < this.vertices.length; i++) this.vertices [i].z = 0;
*/
};
  
// Special cases:

function createRectangle (xCentre, yCentre, halfwidth, halfdepth)
{ return new Polygon (xCentre, yCentre, 
  [ {x:xCentre + halfwidth, y:yCentre + halfdepth, z:0},
    {x:xCentre - halfwidth, y:yCentre + halfdepth, z:0},
    {x:xCentre - halfwidth, y:yCentre - halfdepth, z:0},
    {x:xCentre + halfwidth, y:yCentre - halfdepth, z:0}
  ]);
}

// Vertical rectangle, corner with least coordinates at (x0, y0, z0)
function createWall (x0, y0, z0, width, depth, height)
{ var x1 = x0 + width, y1 = y0 + depth, z1 = z0 + height;
  var wall = new Polygon (x0 + width / 2, y0 + depth / 2,
    [{x:x0, y:y0, z:z0}, {x:x1, y:y1, z:z0}, {x:x1, y:y1, z:z1}, {x:x0, y:y0, z:z1}]);
  wall.centre.z = z0 + height / 2;
  return wall;
};

function createHexagon (xCentre, yCentre, side)
{ var rad60 = 60 * DEG2RAD, ssin = side * Math.sin (rad60), scos = side * Math.cos (rad60);
  return new Polygon (xCentre, yCentre,
  [ {x:xCentre - side, y:yCentre, z:0},
    {x:xCentre - scos, y:yCentre + ssin, z:0},
    {x:xCentre + scos, y:yCentre + ssin, z:0},
    {x:xCentre + side, y:yCentre, z:0},
    {x:xCentre + scos, y:yCentre - ssin, z:0},
    {x:xCentre - scos, y:yCentre - ssin, z:0}
  ]);
}

function createRegularPolygon (xCentre, yCentre, nPoints, radius)
{ var pts = [], aRad = 0, daRad = 2 * Math.PI / nPoints;
  for (var i = 0; i < nPoints; i++)
  { pts.push (new Point3D (xCentre + radius * Math.cos (aRad),
                yCentre + radius * Math.sin (aRad), 0));
    aRad += daRad;
  }
  return new Polygon (xCentre, yCentre, pts);
}

function createStar (xCentre, yCentre, nPoints, innerRadius, outerRadius)
{ var pts = [], aRad = 0, daRad = Math.PI / nPoints;
  for (var i = 0; i < nPoints; i++)
  { pts.push (new Point3D (xCentre + outerRadius * Math.cos (aRad),
                yCentre + outerRadius * Math.sin (aRad), 0));
    aRad += daRad;
    pts.push (new Point3D (xCentre + innerRadius * Math.cos (aRad),
                yCentre + innerRadius * Math.sin (aRad), 0));
    aRad += daRad;
  }
  return new Polygon (xCentre, yCentre, pts);
}