// Copyright (c) Graham Relf, UK, 2020
// www.grelf.net
'use strict';

// Created as a vertical column. Vertices are an array of {x, y} just describing one end (no z)
function Prism (centreX, centreY, centreZ, halfheight, vertices)
{ Shape.call (this);
  this.centre = new Point3D (centreX, centreY, centreZ);
  var nVerts = vertices.length;
  var fillColours = new Array ();
  if (arguments.length > 5) fillColours = arguments [5];
  else for (var i = 0; i < nVerts + 2; i++) { fillColours.push ({r:96, g:96, b:96}); } // grey #666
  this.background = false;
  var nVerts = vertices.length;
  var bottomVerts = new Array (nVerts), topVerts = new Array (nVerts);
  var bottomZ = this.centre.z - halfheight, topZ = this.centre.z + halfheight;
  for (var i = 0; i < nVerts; i++)
  { var vi = vertices [i];
    bottomVerts [i] = new Point3D (vi.x, vi.y, bottomZ);
    this.vertices.push (bottomVerts [i]);
    topVerts [i] = new Point3D (vi.x, vi.y, topZ);
  }
  for (i = 0; i < nVerts; i++) { this.vertices.push (topVerts [i]); }
  this.faces [0] = 
    new Face (this, this.centre.newOffsetPt (0, 0, -halfheight), bottomVerts, fillColours [0]);
  this.faces [1] = 
    new Face (this, this.centre.newOffsetPt (0, 0, halfheight), topVerts, fillColours [1]);
  for (i = 0; i < nVerts; i++)
  { var sideVerts = new Array (4);
    var j = i + 1; if (j >= nVerts) j = 0;
    var meanX = (vertices [i].x + vertices [j].x) / 2; 
    var meanY = (vertices [i].y + vertices [j].y) / 2;
    var faceCentre = new Point3D (meanX, meanY, this.centre.z);
    sideVerts [0] = bottomVerts [i];
    sideVerts [1] = topVerts [i];
    sideVerts [2] = topVerts [j];
    sideVerts [3] = bottomVerts [j];
    this.faces.push (new Face (this, faceCentre, sideVerts, fillColours [i + 2]));
  }
  this.halfheight = halfheight;
  this.freeFall = false;
  this.velocity = new Vector3D (0, 0, 0);
  this.angular = new Vector3D (0, 0, 0);
}

Prism.prototype = Object.create (Shape.prototype);
Prism.prototype.constructor = Prism;
