// Copyright (c) Graham Relf, UK, 2020
// www.grelf.net
'use strict';
const PI10000 = Math.PI * 10000, PI_BY_2 = Math.PI * 0.5, TWO_PI = Math.PI * 2;
const DEG2RAD = Math.PI / 180, RAD2DEG = 180 / Math.PI;

function error (msg)
{ document.getElementById ('info').innerHTML = 
    '<span style="color:#fff;background:#f00;padding:8px;font-weight:bold">' + msg + '</span>';
}

function hint (msg) { document.getElementById ('hint').innerHTML = msg; }

function random (min, max) { return (max - min) * Math.random () + min; }

function distance (xy1, xy2)
{ var dx = xy1.x - xy2.x, dy = xy1.y - xy2.y;
  return Math.sqrt (dx * dx + dy * dy);
}

function distanceSq (xy1, xy2) // sometimes no need for sqrt
{ var dx = xy1.x - xy2.x, dy = xy1.y - xy2.y;
  return dx * dx + dy * dy;
}

function showButtons ()
{ var el = document.getElementById ("buttons");
  if (undefined === el) { error ('HTML must have a <div id="buttons">'); return; }
  var i, html = '';
  for (i = 0; i < buttons.length; i++) { html += makeButton (buttons [i]); }
  el.innerHTML = html;
  for (i = 0; i < buttons.length; i++)
  { document.getElementById (buttons [i].id).addEventListener ('click', buttons [i].func); }
  world.canvas.addEventListener ('click', onClick);
}

function makeButton (b) //Only for showButtons()
{ var s = '<button class="control" id="' + b.id + '" type="button"';
  if (undefined !== b.alt) s += ' title="Alternative key: ' + b.alt + '"'; 
  return s + '>' + b.text + '<br/>' + b.key + '</button>';
}

var buttons = [
{id:"turnLeft", text:"Turn left", key:"&#8592;", alt:'l', func:turnLeft},
{id:"forward", text:"Go forward", key:"&#8593;", alt:'f', func:forward},
{id:"turnRight", text:"Turn right", key:"&#8594;", alt:'r', func:turnRight},
{id:"turnBack", text:"Turn round", key:"&#8595;", alt:'b', func:turnRound},
{id:"down", text:"Go down", key:"-", alt:'d', func:down},
{id:"up", text:"Go up", key:"+", alt:'u', func:up}
];

function forward () { me.forward (me.step); }
function turnLeft () { me.turn (-me.turnRad); }
function turnRight () { me.turn (me.turnRad); }
function turnRound () { me.turn (Math.PI); }
function up () { me.upOrDown (1); }
function down () { me.upOrDown (-1); }

var keys = [
{func:turnLeft, keys:'Left|ArrowLeft|l|L'},
{func:turnRight, keys:'Right|ArrowRight|r|R'},
{func:forward, keys:'Up|ArrowUp|f|F'},
{func:turnRound, keys:'Down|ArrowDown|b|B'},
{func:up, keys:'+|=|u|U'},
{func:down, keys:'-|_|d|D'}
];

function makeKeyTable ()
{ world.keyTable = {};
  for (var i = 0; i < keys.length; i++)
  { var ki = keys [i];
    var ks = ki.keys.split ('|');
    for (var k = 0; k < ks.length; k++)
    { world.keyTable [ks [k]] = ki.func; 
  } }
  window.addEventListener ('keydown', onKeydown, true);
}

function onKeydown (e)
{ e.stopPropagation ();
  if (world.uiLocked) return;
  if (world.keyTable [e.key]) world.keyTable [e.key] (); 
}

function onClick (e)
{ e.stopPropagation ();
  if (world.uiLocked) return;
  var rect = world.canvas.getBoundingClientRect ();
  var scaleX = world.canvas.width / rect.width;
  var scaleY = world.canvas.height / rect.height;
  var mouseX = (e.clientX - rect.left) * scaleX;
  var mouseY = (e.clientY - rect.top) * scaleY;
  var hit = null, dMin = 1000;
  for (var i = 0; i < world.targets.length; i++)
  { var ti = world.targets [i];
    if (mouseX >= ti.minX && mouseX <= ti.maxX && mouseY >= ti.minY && mouseY <= ti.maxY )
    { var d = ti.centre.distance (me.pt);
      if (d < dMin) { hit = ti; dMin = d; }
  } }
  if (null !== hit) hit.response (mouseX, mouseY);
  else hint ('');
}

function fillTriangle (g2, x0, y0, x1, y1, x2, y2)
{ g2.beginPath ();
  g2.moveTo (x0, y0);
  g2.lineTo (x1, y1);
  g2.lineTo (x2, y2);
  g2.closePath ();
  g2.fill ();
}

function strokeLine (g2, x0, y0, x1, y1)
{ g2.beginPath ();
  g2.moveTo (x0, y0);
  g2.lineTo (x1, y1);
  g2.stroke ();
}