// Copyright (c) Graham Relf, UK, 2020
// www.grelf.net
'use strict';

function Vector3D (dx, dy, dz) { this.dx = dx; this.dy = dy; this.dz = dz; }

Vector3D.prototype.toString = function ()
{ return '(' + this.dx.toFixed (1) + ', ' + this.dy.toFixed (1) + ', ' + this.dz.toFixed (1) + ')'; };

Vector3D.prototype.clone = function ()
{ return new Vector3D (this.dx, this.dy, this.dz); };

Vector3D.prototype.magnitude = function ()
{ return Math.sqrt (this.dx * this.dx + this.dy * this.dy + this.dz * this.dz); };

Vector3D.prototype.scale = function (scalar)
{ this.dx *= scalar; this.dy *= scalar; this.dz *= scalar; };

Vector3D.prototype.add = function (otherVec)
{ this.dx += otherVec.dx; this.dy += otherVec.dy; this.dz += otherVec.dz; };

Vector3D.prototype.dot = function (otherVec)
{ return this.dx * otherVec.dx + this.dy * otherVec.dy + this.dz * otherVec.dz; };

Vector3D.prototype.cross = function (otherVec)
{ return new Vector3D (
    this.dy * otherVec.dz - this.dz * otherVec.dy,
    this.dz * otherVec.dx - this.dx * otherVec.dz,
    this.dx * otherVec.dy - this.dy * otherVec.dx);
};

Vector3D.prototype.reverse = function ()
{ this.dx = -this.dx; this.dy = -this.dy; this.dz = -this.dz; };

Vector3D.prototype.rotateZ = function (degs)
{ if (0 === degs) return;
  var rad = degs * DEG2RAD, cos = Math.cos (rad), sin = Math.sin (rad);
  var x0 = this.dx, y0 = this.dy;
  this.dx = x0 * cos + y0 * sin;
  this.dy = -x0 * sin + y0 * cos;
};