// Copyright (c) Graham Relf, UK, 2020
// www.grelf.net
'use strict';

function Viewer (x, y, z, bDegs)
{ this.pt = new Point3D (x, y, z);
  this.bRad = bDegs * DEG2RAD;
  this.FF = 560;
  this.step = 1;
  this.turnRad = 15 * DEG2RAD;
}

Viewer.prototype.toString = function ()
{ return 'Standing at ' + this.pt.toString () + 
    ' facing bearing ' + Math.round (this.bRad * RAD2DEG) + '&deg;'; };

Viewer.prototype.forward = function (step)
{ var dx = step * Math.sin (this.bRad), dy = step * Math.cos (this.bRad);
  var newPt = new Point3D (this.pt.x + dx, this.pt.y + dy, this.pt.z);
  if (newPt.x > world.range || newPt.x < -world.range
   || newPt.y > world.range || newPt.y < -world.range) return;
  for (var i = 0; i < world.solidWalls.length; i++)
  { var swi = world.solidWalls [i];
    if (swi.tooClose (newPt)) 
    { //if (swi.face.getPoint3DDistance (newPt) < swi.face.getPoint3DDistance (this.pt))
      // No - that allows escape through house corner!
      { hint ("Solid wall"); return; } // Blocked
  } }
  for (i = 0; i < world.doors.length; i++)
  { var wdi = world.doors [i];
    if (wdi.closed && wdi.tooClose (newPt)) { hint ("Door closed"); return; }
  }
  this.pt.offset (dx, dy, 0);
  if (this.score) this.score (); // Method may be added when building world
  world.draw ();
};

Viewer.prototype.turn = function (rad)
{ this.bRad += rad;
  if (this.bRad <= -Math.PI) this.bRad += TWO_PI;
  else if (this.bRad > Math.PI) this.bRad -= TWO_PI;
  world.draw ();
};

Viewer.prototype.upOrDown = function (dz) 
{ if (this.pt.z + dz < 5) return; // Some ground tiles missing at lower z
  this.pt.offset (0, 0, dz);
  world.draw ();
};

Viewer.prototype.getDistanceAndBearing = function (pt3d)
{ var da = this.pt.distanceAndBearing (pt3d);
  var dbRad = da.b - this.bRad;
  if (dbRad <= -Math.PI) dbRad += TWO_PI; else if (dbRad > Math.PI) dbRad -= TWO_PI;
  return {d:da.d, dbRad:dbRad};
};

Viewer.prototype.getScreenXYD = function (pt3d)
{ var dx = pt3d.x - this.pt.x, dy = pt3d.y - this.pt.y, d = Math.sqrt (dx * dx + dy * dy);
  var dbRad = Math.atan2 (dx, dy) - this.bRad;
  if (dbRad <= -Math.PI) dbRad += TWO_PI; else if (dbRad > Math.PI) dbRad -= TWO_PI;
  var sinb = Math.sin (dbRad), cosb = Math.cos (dbRad);
  var zz = d * cosb; if (zz < 0.001) zz = 0.001;
  var fRatio = this.FF / zz;
  var sx = fRatio * d * sinb + world.screenWd2; // Relative to screen centre
  var sy = world.screenHt2 - fRatio * (pt3d.z - this.pt.z);
  return {x:sx, y:sy, d:d};
};