// Copyright (c) Graham Relf, UK, 2021
// www.grelf.net
'use strict';

// Has no faces
function Wire (vertices, colour) // Array of {x, y, z}, {r, g, b}
{ Shape.call (this);
  this.colour = colour;
  var nVerts = vertices.length;
  var sumX = 0, sumY = 0, sumZ = 0; // For finding centre
  for (var i = 0; i < nVerts; i++)
  { var vi = vertices [i];
    this.vertices.push (new Point3D (vi.x, vi.y, vi.z));
    sumX += vi.x; sumY += vi.y; sumZ += vi.z;
  }
  this.centre = new Point3D (sumX / nVerts, sumY / nVerts, sumZ / nVerts);
}

Wire.prototype = Object.create (Shape.prototype);
Wire.prototype.constructor = Wire;

Wire.prototype.view = function (me, fogNo) // Override
{ var nVerts = this.vertices.length;
  this.screenPts = [];
  for (var i = 0; i < nVerts; i++)
  { this.screenPts.push (me.getScreenXYD (this.vertices [i])); }
  world.g2.strokeStyle = fog (this.colour.r, this.colour.g, this.colour.b, fogNo);
  world.g2.lineWidth = 1 + 4 / me.pt.distance (this.centre);
  if (2 === nVerts) this.viewLine (0, 1);
  else
  { var dist2s = [];
    for (var i = 1, im1 = 0; i < nVerts; i++, im1++)
    { dist2s.push ({d2:this.screenPts [im1].d + this.screenPts [i].d, im1:im1, i:i}); }
    dist2s.sort (function (a, b) {return a.d2 - b.d2;}); // Ascending distance
    for (var i = dist2s.length - 1; i >= 0; i--)
    { var di = dist2s [i];
      this.viewLine (di.im1, di.i);
  } }
  world.g2.lineWidth = 1;
};

Wire.prototype.viewLine = function (i1, i2)
{ var g2 = world.g2;
  g2.beginPath ();
  g2.moveTo (this.screenPts [i1].x, this.screenPts [i1].y);
  g2.lineTo (this.screenPts [i2].x, this.screenPts [i2].y);
  g2.stroke ();
};
